import collections
import contextlib
import errno
import functools
import glob
import mmap
import operator
import os
import random
import shutil
import tempfile
import time
import uuid

import eventlet
from eventlet import greenthread
from eventlet import tpool
from oslo_config import cfg
#from oslo_log import log as logging
from nova.openstack.common import log as logging
from oslo_serialization import jsonutils

from oslo_utils import units
import six

from nova import exception
from nova.compute import hvtype
#from nova.compute import hv_type
from nova.compute import vm_mode
from nova.compute import power_state
from nova.i18n import _
from nova.i18n import _LE
from nova.i18n import _LI
from nova.i18n import _LW

from nova import context
from nova.virt import driver
from nova.virt.pcenter import host
from nova.virt.pcenter import client
from nova.virt import hardware
from nova.network import neutronv2 as network_api

import traceback
LOG = logging.getLogger(__name__)

pcenter_opts = [
    cfg.IntOpt('version',
               help='Pcenter version'),
    cfg.StrOpt('host_ip',
               help='Rest api url for Pcenter'),
    cfg.StrOpt('pcenter_url',
               help='Rest api url for Pcenter'),
    cfg.StrOpt('verify_cert',
               help='Verify certificates path for Pcenter rest api'),
    cfg.StrOpt('host_cert',
               help='Host certificates path for Pcenter rest api'),
    cfg.StrOpt('pcenter_username',
               help='username for Pcenter rest api'),
    cfg.StrOpt('pcenter_password',
               help='password for Pcenter rest api'),
]

CONF = cfg.CONF
CONF.register_opts(pcenter_opts, 'pcenter')
CONF.import_opt('host', 'nova.netconf')
CONF.import_opt('my_ip', 'nova.netconf')
CONF.import_opt('pcenter_url',
                'nova.virt.pcenter.host',
                group='pcenter')
CONF.import_opt('verify_cert',
                'nova.virt.pcenter.host',
                group='pcenter')
CONF.import_opt('host_ip',
                'nova.virt.pcenter.host',
                group='pcenter')
CONF.import_opt('host_cert',
                'nova.virt.pcenter.host',
                group='pcenter')
CONF.import_opt('pcenter_username',
                'nova.virt.pcenter.host',
                group='pcenter')
CONF.import_opt('pcenter_password',
                'nova.virt.pcenter.host',
                group='pcenter')
CONF.import_opt('my_ip', 'nova.netconf')
MAX_CONSOLE_BYTES = 100 * units.Ki

# The libvirt driver will prefix any disable reason codes with this string.
DISABLE_PREFIX = 'AUTO: '
# Disable reason for the service which was enabled or disabled without reason
DISABLE_REASON_UNDEFINED = None

# Guest config console string
CONSOLE = "console=tty0 console=ttyS0"

# Pcenter Power State
RUNNING = 'Running'
SHUTDOWN = 'Not Activated'
FIRMWARE = 'Open Firmware'
OTHER = 'other'

PCENTER_POWER_STATE = (RUNNING, SHUTDOWN)

PCENTER_POWER_STATE_MAP = {
    RUNNING: power_state.RUNNING,
    SHUTDOWN:power_state.SHUTDOWN,
    FIRMWARE:power_state.RUNNING,
    OTHER: power_state.CRASHED,
}

#Pcenter Network Type
PCENTER_NETWORK_TYPE = ('flat', 'vlan','vxlan')

def patch_tpool_proxy():
    """eventlet.tpool.Proxy doesn't work with old-style class in __str__()
    or __repr__() calls. See bug #962840 for details.
    We perform a monkey patch to replace those two instance methods.
    """
    def str_method(self):
        return str(self._obj)

    def repr_method(self):
        return repr(self._obj)

    tpool.Proxy.__str__ = str_method
    tpool.Proxy.__repr__ = repr_method


patch_tpool_proxy()

class PcenterDriver(driver.ComputeDriver):

    capabilities = {
        "has_imagecache": False,
        "supports_recreate": False,
        "supports_migrate_to_same_host": False
    }

    def __init__(self, virtapi):
        self.virtapi = virtapi
        self._compute_event_callback = None
        self.client = client.PcenterClient(CONF.pcenter.pcenter_url,
                                           CONF.pcenter.pcenter_username,
                                           CONF.pcenter.pcenter_password,
                                           CONF.pcenter.verify_cert,
                                           CONF.pcenter.host_cert)
        self._driver_type = 'pcenter'
        self._host = {}


    def init_host(self, host):
        """Initialize anything that is necessary for the driver to function,
        including catching up with currently running VM's on the given host.
        """
        pass

    def cleanup_host(self, host):
        """Clean up anything that is necessary for the driver gracefully stop,
        including ending remote sessions. This is optional.
        """
        pass

    def get_info(self, instance):
        """Get the current status of an instance, by name (not ID!)

        :param instance: nova.objects.instance.Instance object

        Returns a InstanceInfo object
        """
        # TODO(Vek): Need to pass context in for access to auth_token
        # return self._host.get_vm_info(instance)

        path = ('/vm/%(display_name)s/info'% {
                    'display_name': instance['display_name']})
        response = self.client.get(path)
        status_code = response.status_code
        if status_code != 200 :
            msg = (_(
                'Error from pcenter while get instance %(instance_uuid)s info.'
                'Status code: %(status_code)d.'
                'Reason: %(reason)s ' % {'instance_uuid': instance['uuid'],
                                          'status_code': status_code,
                                          'reason': response.reason}))
            raise exception.NovaException(msg)
        result = response.json()
        if result == None:
            raise exception.NovaException('Response error')
        lpar_state = result.get('status')
        if result.get('code') == "1001" or result.get('code') == "1002":
            raise exception.NovaException('%s' %(result.get('msg')))
        if result.get('code') == "3006":
            raise exception.InstanceNotFound('this vm not found')
        LOG.info(_LI("lpar power state %(lpar_state)s"),
                 {
                     'lpar_state': lpar_state
                 })
        if lpar_state not in PCENTER_POWER_STATE:
            lpar_state = OTHER
        self.data={}
        mem_kb = instance['memory_mb'] * 1024
        num_cpu = instance['vcpus']
        return  {'state': PCENTER_POWER_STATE_MAP.get(lpar_state),
                 'max_mem_kb': mem_kb,
                 'mem_kb': mem_kb,
                 'num_cpu': num_cpu,
                 'cpu_time_ns':0}
    def get_num_instances(self):
        """Return the total number of virtual machines.

        Return the number of virtual machines that the hypervisor knows
        about.

        .. note::

            This implementation works for all drivers, but it is
            not particularly efficient. Maintainers of the virt drivers are
            encouraged to override this method with something more
            efficient.
        """
        return len(self.list_instances())

    def instance_exists(self, instance):
        """Checks existence of an instance on the host.

        :param instance: The instance to lookup

        Returns True if an instance with the supplied ID exists on
        the host, False otherwise.

        .. note::

            This implementation works for all drivers, but it is
            not particularly efficient. Maintainers of the virt drivers are
            encouraged to override this method with something more
            efficient.
        """
        try:
            return instance.uuid in self.list_instance_uuids()
        except NotImplementedError:
            return instance.name in self.list_instances()

    def estimate_instance_overhead(self, instance_info):
        """Estimate the virtualization overhead required to build an instance
        of the given flavor.

        Defaults to zero, drivers should override if per-instance overhead
        calculations are desired.

        :param instance_info: Instance/flavor to calculate overhead for.
        :returns: Dict of estimated overhead values.
        """
        return {'memory_mb': 0}

    def _list_lpar(self):
        path = ('/vm')
        response = self.client.get(path)
        status_code = response.status_code
        if status_code != 200:
            msg = (_(
                'Error from pcenter while get all VMs name on host '
                'Status code: %(status_code)d.'
                'Reason: %(reason)s ' % {'status_code': status_code,
                                          'reason': response.reason}))
            raise exception.NovaException(msg)
        result = response.json()
        if result.get('code') == "1001" or result.get('code') == "1002":
            raise exception.NovaException('%s' % (result.get('msg')))
        return result.get('vms')

    def list_instances(self):
        names = self._list_lpar()
        return names

    def list_instance_uuids(self):
        """Return the UUIDS of all the instances known to the virtualization
        layer, as a list.
        """
        return []

    def rebuild(self, context, instance, image_meta, injected_files,
                admin_password, bdms, detach_block_devices,
                attach_block_devices, network_info=None,
                recreate=False, block_device_info=None,
                preserve_ephemeral=False):
        """Destroy and re-make this instance.

        A 'rebuild' effectively purges all existing data from the system and
        remakes the VM with given 'metadata' and 'personalities'.

        This base class method shuts down the VM, detaches all block devices,
        then spins up the new VM afterwards. It may be overridden by
        hypervisors that need to - e.g. for optimisations, or when the 'VM'
        is actually proxied and needs to be held across the shutdown + spin
        up steps.

        :param context: security context
        :param instance: nova.objects.instance.Instance
                         This function should use the data there to guide
                         the creation of the new instance.
        :param image_meta: image object returned by nova.image.glance that
                           defines the image from which to boot this instance
        :param injected_files: User files to inject into instance.
        :param admin_password: Administrator password to set in instance.
        :param bdms: block-device-mappings to use for rebuild
        :param detach_block_devices: function to detach block devices. See
            nova.compute.manager.ComputeManager:_rebuild_default_impl for
            usage.
        :param attach_block_devices: function to attach block devices. See
            nova.compute.manager.ComputeManager:_rebuild_default_impl for
            usage.
        :param network_info:
           :py:meth:`~nova.network.manager.NetworkManager.get_instance_nw_info`
        :param recreate: True if the instance is being recreated on a new
            hypervisor - all the cleanup of old state is skipped.
        :param block_device_info: Information about block devices to be
                                  attached to the instance.
        :param preserve_ephemeral: True if the default ephemeral storage
                                   partition must be preserved on rebuild
        """
        raise NotImplementedError()

    def spawn(self, context, instance, image_meta, injected_files,
              admin_password, network_info=None, block_device_info=None):
        """Create a new instance/VM/domain on the virtualization platform.

        Once this successfully completes, the instance should be
        running (power_state.RUNNING).

        If this fails, any partial instance should be completely
        cleaned up, and the virtualization platform should be in the state
        that it was before this call began.

        :param context: security context
        :param instance: nova.objects.instance.Instance
                         This function should use the data there to guide
                         the creation of the new instance.
        :param image_meta: image object returned by nova.image.glance that
                           defines the image from which to boot this instance
        :param injected_files: User files to inject into instance.
        :param admin_password: Administrator password to set in instance.
        :param network_info:
           :py:meth:`~nova.network.manager.NetworkManager.get_instance_nw_info`
        :param block_device_info: Information about block devices to be
                                  attached to the instance.
        """
        LOG.error("Discarding event ===================%s", str(instance))
        self._update_network_detail(context, network_info)
        if instance['metadata'].has_key('enableAdminPass') and instance['metadata']['enableAdminPass'] == '1': 
            admin_passwd = admin_password
        else:
            admin_passwd = None 

        lpar_name = instance['display_name']
        vcpu = instance['vcpus']
        memory = instance['memory_mb']
        disk = int(instance['system_metadata']['image_min_disk']) * 1024
        vifs = self.plug_vifs(instance, network_info)
        image_name = image_meta['name']
        nodename = instance.get('node')
        self._host[nodename].create_lpar(lpar_name, vcpu, memory, disk, vifs,
                                         image_name, admin_passwd)
       
        print network_info

    def destroy(self, context, instance, network_info, block_device_info=None,
                destroy_disks=True, migrate_data=None):
        """Destroy the specified instance from the Hypervisor.

        If the instance is not found (for example if networking failed), this
        function should still succeed.  It's probably a good idea to log a
        warning in that case.

        :param context: security context
        :param instance: Instance object as returned by DB layer.
        :param network_info:
           :py:meth:`~nova.network.manager.NetworkManager.get_instance_nw_info`
        :param block_device_info: Information about block devices that should
                                  be detached from the instance.
        :param destroy_disks: Indicates if disks should be destroyed
        :param migrate_data: implementation specific params
        """
        #raise NotImplementedError()
        if instance.task_state == 'resize_reverting' and instance.vm_state == 'resized':
            return
        nodename = instance.get('node')
        self._host[nodename].destroy_lpar(instance['display_name'])
        #pass


    def cleanup(self, context, instance, network_info, block_device_info=None,
                destroy_disks=True, migrate_data=None, destroy_vifs=True):
        """Cleanup the instance resources .

        Instance should have been destroyed from the Hypervisor before calling
        this method.

        :param context: security context
        :param instance: Instance object as returned by DB layer.
        :param network_info:
           :py:meth:`~nova.network.manager.NetworkManager.get_instance_nw_info`
        :param block_device_info: Information about block devices that should
                                  be detached from the instance.
        :param destroy_disks: Indicates if disks should be destroyed
        :param migrate_data: implementation specific params
        """
        pass

    def reboot(self, context, instance, network_info, reboot_type,
               block_device_info=None, bad_volumes_callback=None):
        """Reboot the specified instance.

        After this is called successfully, the instance's state
        goes back to power_state.RUNNING. The virtualization
        platform should ensure that the reboot action has completed
        successfully even in cases in which the underlying domain/vm
        is paused or halted/stopped.

        :param instance: nova.objects.instance.Instance
        :param network_info:
           :py:meth:`~nova.network.manager.NetworkManager.get_instance_nw_info`
        :param reboot_type: Either a HARD or SOFT reboot
        :param block_device_info: Info pertaining to attached volumes
        :param bad_volumes_callback: Function to handle any bad volumes
            encountered
        """
        lpar_name = instance['display_name']
        path = (
        '/vm/%(lpar_name)s/state' % {
            'lpar_name': lpar_name})
        data = {"action": 'reboot'}
        response = self.client.put(path, data)
        status_code = response.status_code
        if response.status_code != 200 :
            msg = (_(
                'Error from pcenter while power off lpar %(lpar_name)s.'
                'Status code: %(status_code)d.'
                'Reason: %(reason)s ' % {'lpar_name': lpar_name,
                                          'status_code': response.status_code,
                                          'reason': response.reason}))
            raise exception.NovaException(msg)
        result = response.json()
        if result.get('code') != '0000':
            raise exception.NovaException('%s' %result.get('msg'))
        return
        #raise NotImplementedError()

    def get_console_pool_info(self, console_type):
        # TODO(Vek): Need to pass context in for access to auth_token
        raise NotImplementedError()

    def get_console_output(self, context, instance):
        """Get console output for an instance

        :param context: security context
        :param instance: nova.objects.instance.Instance
        """
        raise NotImplementedError()

    def get_vnc_console(self, context, instance):
        """Get connection info for a vnc console.

        :param context: security context
        :param instance: nova.objects.instance.Instance

        :returns an instance of console.type.ConsoleVNC
        """
        raise NotImplementedError()

    def get_spice_console(self, context, instance):
        """Get connection info for a spice console.

        :param context: security context
        :param instance: nova.objects.instance.Instance

        :returns an instance of console.type.ConsoleSpice
        """
        raise NotImplementedError()

    def get_rdp_console(self, context, instance):
        """Get connection info for a rdp console.

        :param context: security context
        :param instance: nova.objects.instance.Instance

        :returns an instance of console.type.ConsoleRDP
        """
        raise NotImplementedError()

    def get_serial_console(self, context, instance):
        """Get connection info for a serial console.

        :param context: security context
        :param instance: nova.objects.instance.Instance

        :returns an instance of console.type.ConsoleSerial
        """
        raise NotImplementedError()

    def get_diagnostics(self, instance):
        """Return data about VM diagnostics.

        :param instance: nova.objects.instance.Instance
        """
        # TODO(Vek): Need to pass context in for access to auth_token
        raise NotImplementedError()

    def get_instance_diagnostics(self, instance):
        """Return data about VM diagnostics.

        :param instance: nova.objects.instance.Instance
        """
        raise NotImplementedError()

    def get_all_bw_counters(self, instances):
        """Return bandwidth usage counters for each interface on each
           running VM.

        :param instances: nova.objects.instance.InstanceList
        """
        raise NotImplementedError()

    def get_all_volume_usage(self, context, compute_host_bdms):
        """Return usage info for volumes attached to vms on
           a given host.-
        """
        raise NotImplementedError()

    def get_host_ip_addr(self):
        """Retrieves the IP address of the dom0
        """
        # TODO(Vek): Need to pass context in for access to auth_token
	return CONF.my_ip
    def attach_volume(self, context, connection_info, instance, mountpoint,
                      disk_bus=None, device_type=None, encryption=None):
        """Attach the disk to the instance at mountpoint using info."""
        LOG.info("Discarding event ===================%s", str(connection_info))
        lpar_name = instance['display_name']
	size = connection_info['data']['volumeSize']
	volume_id = connection_info['serial']
	path = ('/vm/addDisk')
	data = {'name': lpar_name,
		'uuid': volume_id,
		'disk': int(size)}
        response = self.client.post(path,data)
        if response.status_code != 200 :
            msg = (_(
                'Error from pcenter while attach volume device %(lpar_name)s.'
                'Status code: %(status_code)d.'
                'Reason: %(reason)s ' % {'lpar_name': lpar_name,
                                          'status_code': response.status_code,
                                          'reason': response.reason}))
            raise exception.NovaException(msg)
        result = response.json()
        if result.get('code') != '0000':
            raise exception.NovaException('%s' %result.get('msg'))
	

    def detach_volume(self, connection_info, instance, mountpoint,
                      encryption=None):
        """Detach the disk attached to the instance."""
        LOG.info("Discarding event ===================%s", str(connection_info))
        lpar_name = instance['display_name']
        volume_id = connection_info['serial']
        path = (
        '/vm/%(lpar_name)s/%(volume_id)s/delDisk' % {
            'lpar_name': lpar_name,
	    'volume_id': volume_id})
        response = self.client.delete(path)
        if response.status_code != 200 :
            msg = (_(
                'Error from pcenter while detach volume device %(lpar_name)s.'
                'Status code: %(status_code)d.'
                'Reason: %(reason)s ' % {'lpar_name': lpar_name,
                                          'status_code': response.status_code,
                                          'reason': response.reason}))
            raise exception.NovaException(msg)
        result = response.json()
        if result.get('code') != '0000':
            raise exception.NovaException('%s' %result.get('msg'))

    def swap_volume(self, old_connection_info, new_connection_info,
                    instance, mountpoint, resize_to):
        """Replace the disk attached to the instance.

        :param instance: nova.objects.instance.Instance
        :param resize_to: This parameter is used to indicate the new volume
                          size when the new volume lager than old volume.
                          And the units is Gigabyte.
        """
        raise NotImplementedError()

    def get_pci_slots_from_xml(self, instance):
        return []

    def get_context(self):
        admin_context = context.RequestContext('cloud_admin',
                                               'admin',is_admin=True)
        return admin_context

    def attach_interface(self, instance, image_meta, vif):
        """Attach an interface to the instance.

        :param instance: nova.objects.instance.Instance
        """
        from nova import context as context_api
        context = context_api.get_admin_context()
        #context = self.get_context()
        lpar_name = instance['display_name']
        vif = self._update_network_detail_interface(context, vif)
	uuid = vif['id']
	vif_info = {'ip' : 0,
                    'mask' : 0,
                    'gateway' : 0,
                    'mac' : 0}
        vlan = 0
        subnets = vif['network']['subnets']
        if len(subnets) != 0:
            subnet = subnets[0]
            vif_info['ip'] = subnet['ips'][0]['address']
	    cidr= subnet['cidr']
	    vif_info['mask'] = cidr.split('/')[1]

            if subnet['gateway']['address']:
                vif_info['gateway'] = subnet['gateway']['address']
            if vif['address']:
                vif_info['mac'] = vif['address']
            vlan = vif['network']['vlan']
	
	vifs = '%s/%s/%s/%s/%s' % (vif_info['ip'],vif_info['mask'],
	                           vif_info['gateway'],vlan,vif_info['mac'])
        LOG.info("Discarding event ===================%s", str(vifs))
        path = ('/vm/addEth')
	data = {'name': lpar_name,
		'uuid': uuid,
		'networks': vifs}
        response = self.client.post(path,data)
        if response.status_code != 200 :
            msg = (_(
                'Error from pcenter while attach network adapter device %(lpar_name)s.'
                'Status code: %(status_code)d.'
                'Reason: %(reason)s ' % {'lpar_name': lpar_name,
                                          'status_code': response.status_code,
                                          'reason': response.reason}))
            raise exception.NovaException(msg)
        result = response.json()
        if result.get('code') != '0000':
            raise exception.NovaException('%s' %result.get('msg'))

    def detach_interface(self, instance, vif):
        """Detach an interface from the instance.

        :param instance: nova.objects.instance.Instance
        """
        lpar_name = instance['display_name']
	uuid = vif['id']
        LOG.info("Discarding event ===================%s", str(vif))
        path = (
        '/vm/%(lpar_name)s/%(uuid)s/delEth' % {
            'lpar_name': lpar_name,
	    'uuid': uuid})
        response = self.client.delete(path)
        if response.status_code != 200 :
            msg = (_(
                'Error from pcenter while detach network adapter device %(lpar_name)s.'
                'Status code: %(status_code)d.'
                'Reason: %(reason)s ' % {'lpar_name': lpar_name,
                                          'status_code': response.status_code,
                                          'reason': response.reason}))
            raise exception.NovaException(msg)
        result = response.json()
        if result.get('code') != '0000':
            raise exception.NovaException('%s' %result.get('msg'))

    def migrate_disk_and_power_off(self, context, instance, dest,
                                   flavor, network_info,
                                   block_device_info=None,
                                   timeout=0, retry_interval=0):
        """Transfers the disk of a running instance in multiple phases, turning
        off the instance before the end.

        :param instance: nova.objects.instance.Instance
        :param timeout: time to wait for GuestOS to shutdown
        :param retry_interval: How often to signal guest while
                               waiting for it to shutdown
        """
	return None

    def snapshot(self, context, instance, image_id, update_task_state):
        """Snapshots the specified instance.

        :param context: security context
        :param instance: nova.objects.instance.Instance
        :param image_id: Reference to a pre-created image that will
                         hold the snapshot.
        """
        raise NotImplementedError()

    def post_interrupted_snapshot_cleanup(self, context, instance):
        """Cleans up any resources left after an interrupted snapshot.

        :param context: security context
        :param instance: nova.objects.instance.Instance
        """
        pass
    def change_cpu(self,lpar_name,vcpus):
	path = ("/vm/updateCpu")
	data = {'name': lpar_name,
	       	'cpu': vcpus / float(10)}
        response = self.client.put(path, data)
        if response.status_code != 200 :
            msg = (_(
                'Error from pcenter while change vcpus lpar  %(lpar_name)s.'
                'Status code: %(status_code)d.'
                'Reason: %(reason)s ' % {'lpar_name': lpar_name,
                                          'status_code': response.status_code,
                                          'reason': response.reason}))
            raise exception.NovaException(msg)
        result = response.json()
        if result.get('code') != '0000':
            raise exception.NovaException('%s' %result.get('msg'))

    def change_mem(self,lpar_name,mem):
	path = ("/vm/updateMemory")
	data = {'name': lpar_name,
	       	'memory': mem}
        response = self.client.put(path, data)
        if response.status_code != 200 :
            msg = (_(
                'Error from pcenter while change memory lpar  %(lpar_name)s.'
                'Status code: %(status_code)d.'
                'Reason: %(reason)s ' % {'lpar_name': lpar_name,
                                          'status_code': response.status_code,
                                          'reason': response.reason}))
            raise exception.NovaException(msg)
        result = response.json()
        if result.get('code') != '0000':
            raise exception.NovaException('%s' %result.get('msg'))
	
    def change_spec(self, lpar_name, cpu, mem, disk=None, disk_uuid=None):
        path = ("/vm/updateSpec")
        data = {'name': lpar_name,
               	'cpu': cpu/float(10),
                'memory': mem,
                'disk': 0,
                'disk_uuid': 0}
        response = self.client.put(path, data)
        if response.status_code != 200 :
            msg = (_(
                'Error from pcenter while change vcpus lpar  %(lpar_name)s.'
                'Status code: %(status_code)d.'
                'Reason: %(reason)s ' % {'lpar_name': lpar_name,
                                          'status_code': response.status_code,
                                          'reason': response.reason}))
            raise exception.NovaException(msg)
        result = response.json()
        if result.get('code') != '0000':
            raise exception.NovaException('%s' %result.get('msg'))
	
    def finish_migration(self, context, migration, instance, disk_info,
                         network_info, image_meta, resize_instance,
                         block_device_info=None, power_on=True):
        """Completes a resize.

        :param context: the context for the migration/resize
        :param migration: the migrate/resize information
        :param instance: nova.objects.instance.Instance being migrated/resized
        :param disk_info: the newly transferred disk information
        :param network_info:
           :py:meth:`~nova.network.manager.NetworkManager.get_instance_nw_info`
        :param image_meta: image object returned by nova.image.glance that
                           defines the image from which this instance
                           was created
        :param resize_instance: True if the instance is being resized,
                                False otherwise
        :param block_device_info: instance volume block device info
        :param power_on: True if the instance should be powered on, False
                         otherwise
        """
	lpar_name = instance['display_name']
	vcpus = instance.vcpus
	mem = instance.memory_mb
        disk = instance.root_gb
	#self.change_mem(lparname,mem)
	#self.change_cpu(lparname,vcpus)
	self.change_spec(lpar_name,vcpus,mem)

    def confirm_migration(self, migration, instance, network_info):
        """Confirms a resize, destroying the source VM.

        :param instance: nova.objects.instance.Instance
        """
        # TODO(Vek): Need to pass context in for access to auth_token
	pass

    def finish_revert_migration(self, context, instance, network_info,
                                block_device_info=None, power_on=True):
        """Finish reverting a resize.

        :param context: the context for the finish_revert_migration
        :param instance: nova.objects.instance.Instance being migrated/resized
        :param network_info:
           :py:meth:`~nova.network.manager.NetworkManager.get_instance_nw_info`
        :param block_device_info: instance volume block device info
        :param power_on: True if the instance should be powered on, False
                         otherwise
        """
	lpar_name = instance['display_name']
	vcpus = instance.vcpus
	mem = instance.memory_mb
        disk = instance.root_gb
	self.change_spec(lpar_name,vcpus,mem)
	#self.change_mem(lparname,mem)
	#self.change_cpu(lparname,vcpus)
	

    def pause(self, instance):
        """Pause the specified instance.

        :param instance: nova.objects.instance.Instance
        """
        # TODO(Vek): Need to pass context in for access to auth_token
        raise NotImplementedError()

    def unpause(self, instance):
        """Unpause paused VM instance.

        :param instance: nova.objects.instance.Instance
        """
        # TODO(Vek): Need to pass context in for access to auth_token
        raise NotImplementedError()

    def suspend(self, context, instance):
        """suspend the specified instance.

        :param context: the context for the suspend
        :param instance: nova.objects.instance.Instance
        """
        raise NotImplementedError()

    def resume(self, context, instance, network_info, block_device_info=None):
        """resume the specified instance.

        :param context: the context for the resume
        :param instance: nova.objects.instance.Instance being resumed
        :param network_info:
           :py:meth:`~nova.network.manager.NetworkManager.get_instance_nw_info`
        :param block_device_info: instance volume block device info
        """
        raise NotImplementedError()

    def resume_state_on_host_boot(self, context, instance, network_info,
                                  block_device_info=None):
        """resume guest state when a host is booted.

        :param instance: nova.objects.instance.Instance
        """
        return
        #raise NotImplementedError()

    def rescue(self, context, instance, network_info, image_meta,
               rescue_password):
        """Rescue the specified instance.

        :param instance: nova.objects.instance.Instance
        """
        raise NotImplementedError()

    def set_bootable(self, instance, is_bootable):
        """Set the ability to power on/off an instance.

        :param instance: nova.objects.instance.Instance
        """
        raise NotImplementedError()

    def unrescue(self, instance, network_info):
        """Unrescue the specified instance.

        :param instance: nova.objects.instance.Instance
        """
        # TODO(Vek): Need to pass context in for access to auth_token
        raise NotImplementedError()

    def power_off(self, instance, timeout=0, retry_interval=0):
        """Power off the specified instance.

        :param instance: nova.objects.instance.Instance
        :param timeout: time to wait for GuestOS to shutdown
        :param retry_interval: How often to signal guest while
                               waiting for it to shutdown
        """
        lpar_name = instance['display_name']
        path = (
        '/vm/%(lpar_name)s/state' % {
            'lpar_name': lpar_name})
        data = {"action": 'off'}
        response = self.client.put(path, data)
        status_code = response.status_code
        if response.status_code != 200 :
            msg = (_(
                'Error from pcenter while power off lpar %(lpar_name)s.'
                'Status code: %(status_code)d.'
                'Reason: %(reason)s ' % {'lpar_name': lpar_name,
                                          'status_code': response.status_code,
                                          'reason': response.reason}))
            raise exception.NovaException(msg)
        result = response.json()
        if result.get('code') == '3002':
            return
        if result.get('code') != '0000':
            raise exception.NovaException('%s' %result.get('msg'))


    def power_on(self, context, instance, network_info,
                 block_device_info=None):
        """Power on the specified instance.

        :param instance: nova.objects.instance.Instance
        """
        lpar_name = instance['display_name']
        path = (
        '/vm/%(lpar_name)s/state' % {
            'lpar_name': lpar_name})
        data = {"action": 'on'}
        response = self.client.put(path, data)
        status_code = response.status_code
        if response.status_code != 200 :
            msg = (_(
                'Error from pcenter while power on lpar %(lpar_name)s.'
                'Status code: %(status_code)d.'
                'Reason: %(reason)s ' % {'lpar_name': lpar_name,
                                          'status_code': response.status_code,
                                          'reason': response.reason}))
            raise exception.NovaException(msg)
        result = response.json()
        if result.get('code') == '3001':
            return
        if result.get('code') != '0000':
            raise exception.NovaException('%s' %result.get('msg'))

    def soft_delete(self, instance):
        """Soft delete the specified instance.

        :param instance: nova.objects.instance.Instance
        """
        raise NotImplementedError()

    def restore(self, instance):
        """Restore the specified instance.

        :param instance: nova.objects.instance.Instance
        """
        raise NotImplementedError()

    def get_available_resource(self, nodename):
        """Retrieve resource information.

        This method is called when nova-compute launches, and
        as part of a periodic task that records the results in the DB.

        :param nodename:
            node which the caller want to get resources from
            a driver that manages only one node can safely ignore this
        :returns: Dictionary describing resources
        """
        host_info = self._host[nodename].update_host_info()
        data = {}
        cpu_info = self._host[nodename].get_cpu_info()
        disk_info = self._host[nodename].get_local_gb_info()
        data["supported_instances"] = jsonutils.dumps(
            self._get_instance_capabilities(cpu_info['arch']))
        data["vcpus"] = self._host[nodename].get_vcpu_total() * 10
        data["memory_mb"] = self._host[nodename].get_memory_mb_total()
        data["local_gb"] = disk_info['total']
        data["vcpus_used"] = self._host[nodename].get_vcpu_used() * 10
        data["memory_mb_used"] = self._host[nodename].get_memory_mb_used()
        data["local_gb_used"] = disk_info['used']
        data["hypervisor_type"] = self._driver_type
        data["hypervisor_version"] = self._get_version()
        data["hypervisor_hostname"] = nodename
        
        data['numa_topology'] = None
        data["cpu_info"] = jsonutils.dumps(cpu_info)
        data['disk_available_least'] = disk_info['free']
        LOG.info(_LI("lingcloudpower %(data)s"),
            {
                'data':jsonutils.dumps(data)
            })
        return data

    def pre_live_migration(self, context, instance, block_device_info,
                           network_info, disk_info, migrate_data=None):
        """Prepare an instance for live migration

        :param context: security context
        :param instance: nova.objects.instance.Instance object
        :param block_device_info: instance block device information
        :param network_info: instance network information
        :param disk_info: instance disk information
        :param migrate_data: implementation specific data dict.
        """
        raise NotImplementedError()

    def live_migration(self, context, instance, dest,
                       post_method, recover_method, block_migration=False,
                       migrate_data=None):
        """Live migration of an instance to another host.

        :param context: security context
        :param instance:
            nova.db.sqlalchemy.models.Instance object
            instance object that is migrated.
        :param dest: destination host
        :param post_method:
            post operation method.
            expected nova.compute.manager._post_live_migration.
        :param recover_method:
            recovery method when any exception occurs.
            expected nova.compute.manager._rollback_live_migration.
        :param block_migration: if true, migrate VM disk.
        :param migrate_data: implementation specific params.

        """
        raise NotImplementedError()

    def rollback_live_migration_at_destination(self, context, instance,
                                               network_info,
                                               block_device_info,
                                               destroy_disks=True,
                                               migrate_data=None):
        """Clean up destination node after a failed live migration.

        :param context: security context
        :param instance: instance object that was being migrated
        :param network_info: instance network information
        :param block_device_info: instance block device information
        :param destroy_disks:
            if true, destroy disks at destination during cleanup
        :param migrate_data: implementation specific params

        """
        raise NotImplementedError()

    def post_live_migration(self, context, instance, block_device_info,
                            migrate_data=None):
        """Post operation of live migration at source host.

        :param context: security context
        :instance: instance object that was migrated
        :block_device_info: instance block device information
        :param migrate_data: if not None, it is a dict which has data
        """
        pass

    def post_live_migration_at_source(self, context, instance, network_info):
        """Unplug VIFs from networks at source.

        :param context: security context
        :param instance: instance object reference
        :param network_info: instance network information
        """
        raise NotImplementedError(_("Hypervisor driver does not support "
                                    "post_live_migration_at_source method"))

    def post_live_migration_at_destination(self, context, instance,
                                           network_info,
                                           block_migration=False,
                                           block_device_info=None):
        """Post operation of live migration at destination host.

        :param context: security context
        :param instance: instance object that is migrated
        :param network_info: instance network information
        :param block_migration: if true, post operation of block_migration.
        """
        raise NotImplementedError()

    def check_instance_shared_storage_local(self, context, instance):
        """Check if instance files located on shared storage.

        This runs check on the destination host, and then calls
        back to the source host to check the results.

        :param context: security context
        :param instance: nova.objects.instance.Instance object
        """
        raise NotImplementedError()

    def check_instance_shared_storage_remote(self, context, data):
        """Check if instance files located on shared storage.

        :param context: security context
        :param data: result of check_instance_shared_storage_local
        """
        raise NotImplementedError()

    def check_instance_shared_storage_cleanup(self, context, data):
        """Do cleanup on host after check_instance_shared_storage calls

        :param context: security context
        :param data: result of check_instance_shared_storage_local
        """
        pass

    def check_can_live_migrate_destination(self, context, instance,
                                           src_compute_info, dst_compute_info,
                                           block_migration=False,
                                           disk_over_commit=False):
        """Check if it is possible to execute live migration.

        This runs checks on the destination host, and then calls
        back to the source host to check the results.

        :param context: security context
        :param instance: nova.db.sqlalchemy.models.Instance
        :param src_compute_info: Info about the sending machine
        :param dst_compute_info: Info about the receiving machine
        :param block_migration: if true, prepare for block migration
        :param disk_over_commit: if true, allow disk over commit
        :returns: a dict containing migration info (hypervisor-dependent)
        """
        raise NotImplementedError()

    def check_can_live_migrate_destination_cleanup(self, context,
                                                   dest_check_data):
        """Do required cleanup on dest host after check_can_live_migrate calls

        :param context: security context
        :param dest_check_data: result of check_can_live_migrate_destination
        """
        raise NotImplementedError()

    def check_can_live_migrate_source(self, context, instance,
                                      dest_check_data, block_device_info=None):
        """Check if it is possible to execute live migration.

        This checks if the live migration can succeed, based on the
        results from check_can_live_migrate_destination.

        :param context: security context
        :param instance: nova.db.sqlalchemy.models.Instance
        :param dest_check_data: result of check_can_live_migrate_destination
        :param block_device_info: result of _get_instance_block_device_info
        :returns: a dict containing migration info (hypervisor-dependent)
        """
        raise NotImplementedError()

    def get_instance_disk_info(self, instance,
                               block_device_info=None):
        """Retrieve information about actual disk sizes of an instance.

        :param instance: nova.objects.Instance
        :param block_device_info:
            Optional; Can be used to filter out devices which are
            actually volumes.
        :return:
            json strings with below format::

                "[{'path':'disk',
                   'type':'raw',
                   'virt_disk_size':'10737418240',
                   'backing_file':'backing_file',
                   'disk_size':'83886080'
                   'over_committed_disk_size':'10737418240'},
                   ...]"
        """
        raise NotImplementedError()

    def refresh_security_group_rules(self, security_group_id):
        """This method is called after a change to security groups.

        All security groups and their associated rules live in the datastore,
        and calling this method should apply the updated rules to instances
        running the specified security group.

        An error should be raised if the operation cannot complete.

        """
        # TODO(Vek): Need to pass context in for access to auth_token
        raise NotImplementedError()

    def refresh_security_group_members(self, security_group_id):
        """This method is called when a security group is added to an instance.

        This message is sent to the virtualization drivers on hosts that are
        running an instance that belongs to a security group that has a rule
        that references the security group identified by `security_group_id`.
        It is the responsibility of this method to make sure any rules
        that authorize traffic flow with members of the security group are
        updated and any new members can communicate, and any removed members
        cannot.

        Scenario:
            * we are running on host 'H0' and we have an instance 'i-0'.
            * instance 'i-0' is a member of security group 'speaks-b'
            * group 'speaks-b' has an ingress rule that authorizes group 'b'
            * another host 'H1' runs an instance 'i-1'
            * instance 'i-1' is a member of security group 'b'

            When 'i-1' launches or terminates we will receive the message
            to update members of group 'b', at which time we will make
            any changes needed to the rules for instance 'i-0' to allow
            or deny traffic coming from 'i-1', depending on if it is being
            added or removed from the group.

        In this scenario, 'i-1' could just as easily have been running on our
        host 'H0' and this method would still have been called.  The point was
        that this method isn't called on the host where instances of that
        group are running (as is the case with
        :py:meth:`refresh_security_group_rules`) but is called where references
        are made to authorizing those instances.

        An error should be raised if the operation cannot complete.

        """
        # TODO(Vek): Need to pass context in for access to auth_token
        raise NotImplementedError()

    def refresh_provider_fw_rules(self):
        """This triggers a firewall update based on database changes.

        When this is called, rules have either been added or removed from the
        datastore.  You can retrieve rules with
        :py:meth:`nova.db.provider_fw_rule_get_all`.

        Provider rules take precedence over security group rules.  If an IP
        would be allowed by a security group ingress rule, but blocked by
        a provider rule, then packets from the IP are dropped.  This includes
        intra-project traffic in the case of the allow_project_net_traffic
        flag for the libvirt-derived classes.

        """
        # TODO(Vek): Need to pass context in for access to auth_token
        raise NotImplementedError()

    def refresh_instance_security_rules(self, instance):
        """Refresh security group rules

        Gets called when an instance gets added to or removed from
        the security group the instance is a member of or if the
        group gains or loses a rule.
        """
        raise NotImplementedError()

    def reset_network(self, instance):
        """reset networking for specified instance."""
        # TODO(Vek): Need to pass context in for access to auth_token
        pass

    def ensure_filtering_rules_for_instance(self, instance, network_info):
        """Setting up filtering rules and waiting for its completion.

        To migrate an instance, filtering rules to hypervisors
        and firewalls are inevitable on destination host.
        ( Waiting only for filtering rules to hypervisor,
        since filtering rules to firewall rules can be set faster).

        Concretely, the below method must be called.
        - setup_basic_filtering (for nova-basic, etc.)
        - prepare_instance_filter(for nova-instance-instance-xxx, etc.)

        to_xml may have to be called since it defines PROJNET, PROJMASK.
        but libvirt migrates those value through migrateToURI(),
        so , no need to be called.

        Don't use thread for this method since migration should
        not be started when setting-up filtering rules operations
        are not completed.

        :param instance: nova.objects.instance.Instance object

        """
        # TODO(Vek): Need to pass context in for access to auth_token
        raise NotImplementedError()

    def filter_defer_apply_on(self):
        """Defer application of IPTables rules."""
        pass

    def filter_defer_apply_off(self):
        """Turn off deferral of IPTables rules and apply the rules now."""
        pass

    def unfilter_instance(self, instance, network_info):
        """Stop filtering instance."""
        # TODO(Vek): Need to pass context in for access to auth_token
        raise NotImplementedError()

    def set_admin_password(self, instance, new_pass):
        """Set the root password on the specified instance.

        :param instance: nova.objects.instance.Instance
        :param new_pass: the new password
        """
        raise NotImplementedError()

    def inject_file(self, instance, b64_path, b64_contents):
        """Writes a file on the specified instance.

        The first parameter is an instance of nova.compute.service.Instance,
        and so the instance is being specified as instance.name. The second
        parameter is the base64-encoded path to which the file is to be
        written on the instance; the third is the contents of the file, also
        base64-encoded.

        NOTE(russellb) This method is deprecated and will be removed once it
        can be removed from nova.compute.manager.
        """
        # TODO(Vek): Need to pass context in for access to auth_token
        raise NotImplementedError()

    def change_instance_metadata(self, context, instance, diff):
        """Applies a diff to the instance metadata.

        This is an optional driver method which is used to publish
        changes to the instance's metadata to the hypervisor.  If the
        hypervisor has no means of publishing the instance metadata to
        the instance, then this method should not be implemented.

        :param context: security context
        :param instance: nova.objects.instance.Instance
        """
        pass

    def inject_network_info(self, instance, nw_info):
        """inject network info for specified instance."""
        # TODO(Vek): Need to pass context in for access to auth_token
        pass

    def poll_rebooting_instances(self, timeout, instances):
        """Poll for rebooting instances

        :param timeout: the currently configured timeout for considering
                        rebooting instances to be stuck
        :param instances: instances that have been in rebooting state
                          longer than the configured timeout
        """
        # TODO(Vek): Need to pass context in for access to auth_token
        raise NotImplementedError()

    def host_power_action(self, action):
        """Reboots, shuts down or powers up the host."""
        raise NotImplementedError()

    def host_maintenance_mode(self, host, mode):
        """Start/Stop host maintenance window. On start, it triggers
        guest VMs evacuation.
        """
        raise NotImplementedError()

    def set_host_enabled(self, enabled):
        """Sets the specified host's ability to accept new instances."""
        # TODO(Vek): Need to pass context in for access to auth_token
        raise NotImplementedError()

    def get_host_uptime(self):
        """Returns the result of calling "uptime" on the target host."""
        # TODO(Vek): Need to pass context in for access to auth_token
        raise NotImplementedError()
    def get_network_seg(self, id):
	from nova.network.neutronv2 import api as neutronapi
	from nova import context as context_api
	netapi=neutronapi.API()
	context = context_api.get_admin_context()
	network=netapi.get_customed_by_h3c(context, id)
	has_seg_id = network.has_key('provider:segmentation_id')
	if has_seg_id:
	    return network['provider:segmentation_id']

    def plug_vifs(self, instance, network_info):
        """Plug VIFs into networks.

        :param instance: nova.objects.instance.Instance
        """
        # TODO(Vek): Need to pass context in for access to auth_token
        network_plugs=[]
        mac_address = 0
        vlan = 0
        for network in network_info:
            subnets = network['network']['subnets']
            if network['address']:
                mac_address = ''.join(network['address'].split(':'))
            if 'vlan' in network['network'].keys():
                vlan = network['network']['vlan']
            self._plug_subnets(network_plugs, subnets,vlan, mac_address)
        vifs = ','.join(network_plugs)
        return vifs

    def _plug_subnets(self, network_plugs, subnets, vlan=0, mac_address=0):
        ip_address = 0;
        netmask = 0;
        gateway = 0;
        for subnet in subnets:
            ips = subnet['ips']
            cidr = subnet['cidr']
            netmask = cidr.split('/')[1]
            if subnet['gateway']['address']:
                 gateway_addr = subnet['gateway']['address']
            ip_address = ips[0]['address']
            address_item = '%s/%s/%s/%s/%s' % (ip_address, netmask, 
					       gateway_addr, vlan, mac_address)
            network_plugs.append(address_item)

    def unplug_vifs(self, instance, network_info):
        """Unplug VIFs from networks.

        :param instance: nova.objects.instance.Instance
        """
        pass

    def get_host_cpu_stats(self):
        """Get the currently known host CPU stats.

        :returns: a dict containing the CPU stat info, eg:

            | {'kernel': kern,
            |  'idle': idle,
            |  'user': user,
            |  'iowait': wait,
            |   'frequency': freq},

                  where kern and user indicate the cumulative CPU time
                  (nanoseconds) spent by kernel and user processes
                  respectively, idle indicates the cumulative idle CPU time
                  (nanoseconds), wait indicates the cumulative I/O wait CPU
                  time (nanoseconds), since the host is booting up; freq
                  indicates the current CPU frequency (MHz). All values are
                  long integers.

        """
        raise NotImplementedError()

    def block_stats(self, instance, disk_id):
        """Return performance counters associated with the given disk_id on the
        given instance.  These are returned as [rd_req, rd_bytes, wr_req,
        wr_bytes, errs], where rd indicates read, wr indicates write, req is
        the total number of I/O requests made, bytes is the total number of
        bytes transferred, and errs is the number of requests held up due to a
        full pipeline.

        All counters are long integers.

        This method is optional.  On some platforms (e.g. XenAPI) performance
        statistics can be retrieved directly in aggregate form, without Nova
        having to do the aggregation.  On those platforms, this method is
        unused.

        Note that this function takes an instance ID.
        """
        raise NotImplementedError()

    def deallocate_networks_on_reschedule(self, instance):
        """Does the driver want networks deallocated on reschedule?"""
        return False

    def macs_for_instance(self, instance):
        """What MAC addresses must this instance have?

        Some hypervisors (such as bare metal) cannot do freeform virtualisation
        of MAC addresses. This method allows drivers to return a set of MAC
        addresses that the instance is to have. allocate_for_instance will take
        this into consideration when provisioning networking for the instance.

        Mapping of MAC addresses to actual networks (or permitting them to be
        freeform) is up to the network implementation layer. For instance,
        with openflow switches, fixed MAC addresses can still be virtualised
        onto any L2 domain, with arbitrary VLANs etc, but regular switches
        require pre-configured MAC->network mappings that will match the
        actual configuration.

        Most hypervisors can use the default implementation which returns None.
        Hypervisors with MAC limits should return a set of MAC addresses, which
        will be supplied to the allocate_for_instance call by the compute
        manager, and it is up to that call to ensure that all assigned network
        details are compatible with the set of MAC addresses.

        This is called during spawn_instance by the compute manager.

        :return: None, or a set of MAC ids (e.g. set(['12:34:56:78:90:ab'])).
            None means 'no constraints', a set means 'these and only these
            MAC addresses'.
        """
        return None

    def dhcp_options_for_instance(self, instance):
        """Get DHCP options for this instance.

        Some hypervisors (such as bare metal) require that instances boot from
        the network, and manage their own TFTP service. This requires passing
        the appropriate options out to the DHCP service. Most hypervisors can
        use the default implementation which returns None.

        This is called during spawn_instance by the compute manager.

        Note that the format of the return value is specific to Quantum
        client API.

        :return: None, or a set of DHCP options, eg:

             |    [{'opt_name': 'bootfile-name',
             |      'opt_value': '/tftpboot/path/to/config'},
             |     {'opt_name': 'server-ip-address',
             |      'opt_value': '1.2.3.4'},
             |     {'opt_name': 'tftp-server',
             |      'opt_value': '1.2.3.4'}
             |    ]

        """
        return None

    def manage_image_cache(self, context, all_instances):
        """Manage the driver's local image cache.

        Some drivers chose to cache images for instances on disk. This method
        is an opportunity to do management of that cache which isn't directly
        related to other calls into the driver. The prime example is to clean
        the cache and remove images which are no longer of interest.

        :param all_instances: nova.objects.instance.InstanceList
        """
        pass

    def add_to_aggregate(self, context, aggregate, host, **kwargs):
        """Add a compute host to an aggregate."""
        # NOTE(jogo) Currently only used for XenAPI-Pool
        raise NotImplementedError()

    def remove_from_aggregate(self, context, aggregate, host, **kwargs):
        """Remove a compute host from an aggregate."""
        raise NotImplementedError()

    def undo_aggregate_operation(self, context, op, aggregate,
                                  host, set_error=True):
        """Undo for Resource Pools."""
        raise NotImplementedError()

    def get_volume_connector(self, instance):
        """Get connector information for the instance for attaching to volumes.

        Connector information is a dictionary representing the ip of the
        machine that will be making the connection, the name of the iscsi
        initiator and the hostname of the machine as follows::

            {
                'ip': ip,
                'initiator': initiator,
                'host': hostname
            }

        """
	
	return {'ip': '192.168.137.233'}

    def _refresh_cache(self):
        path = ('/host')
        response = self.client.get(path)
        if response.status_code != 200 or response.json().get('code') != '0000':
            msg = (_(
                'Error from pcenter while get available nodes.'
                'Status code: %(status_code)d.'
                'error: %(error)s.'
                'Reason: %(reason)s ' % {'status_code': response.status_code,
                                         'error': response.json().get('msg'),
                                         'reason': response.reason}))
            raise exception.NovaException(msg)
        nodes = [ node for node in response.json().get('hosts')]
        host_cash = {}
        for node in nodes:
            host_cash[node] = host.Host(node, self.client)
        self._host = host_cash

    def get_available_nodes(self, refresh=False):
        self._refresh_cache()
        return [v.hostname for k, v in six.iteritems(self._host)]

    def node_is_available(self, nodename):
        """Return whether this compute service manages a particular node."""
        if nodename in self.get_available_nodes():
            return True
        # Refresh and check again.
        return nodename in self.get_available_nodes(refresh=True)

    def get_per_instance_usage(self):
        """Get information about instance resource usage.

        :returns: dict of  nova uuid => dict of usage info
        """
        return {}

    def instance_on_disk(self, instance):
        """Checks access of instance files on the host.

        :param instance: nova.objects.instance.Instance to lookup

        Returns True if files of an instance with the supplied ID accessible on
        the host, False otherwise.

        .. note::
            Used in rebuild for HA implementation and required for validation
            of access to instance shared disk files
        """
        return False

    def register_event_listener(self, callback):
        """Register a callback to receive events.

        Register a callback to receive asynchronous event
        notifications from hypervisors. The callback will
        be invoked with a single parameter, which will be
        an instance of the nova.virt.event.Event class.
        """

        self._compute_event_callback = callback

    def emit_event(self, event):
        """Dispatches an event to the compute manager.

        Invokes the event callback registered by the
        compute manager to dispatch the event. This
        must only be invoked from a green thread.
        """

        if not self._compute_event_callback:
            LOG.debug("Discarding event %s", str(event))
            return

        try:
            LOG.debug("Emitting event %s", str(event))
            self._compute_event_callback(event)
        except Exception as ex:
            LOG.error(_LE("Exception dispatching event %(event)s: %(ex)s"),
                      {'event': event, 'ex': ex})

    def delete_instance_files(self, instance):
        """Delete any lingering instance files for an instance.

        :param instance: nova.objects.instance.Instance
        :returns: True if the instance was deleted from disk, False otherwise.
        """
        return True

    @property
    def need_legacy_block_device_info(self):
        """Tell the caller if the driver requires legacy block device info.

        Tell the caller whether we expect the legacy format of block
        device info to be passed in to methods that expect it.
        """
        return True

    def volume_snapshot_create(self, context, instance, volume_id,
                               create_info):
        """Snapshots volumes attached to a specified instance.

        :param context: request context
        :param instance: nova.objects.instance.Instance that has the volume
               attached
        :param volume_id: Volume to be snapshotted
        :param create_info: The data needed for nova to be able to attach
               to the volume.  This is the same data format returned by
               Cinder's initialize_connection() API call.  In the case of
               doing a snapshot, it is the image file Cinder expects to be
               used as the active disk after the snapshot operation has
               completed.  There may be other data included as well that is
               needed for creating the snapshot.
        """
        raise NotImplementedError()

    def volume_snapshot_delete(self, context, instance, volume_id,
                               snapshot_id, delete_info):
        """Snapshots volumes attached to a specified instance.

        :param context: request context
        :param instance: nova.objects.instance.Instance that has the volume
               attached
        :param volume_id: Attached volume associated with the snapshot
        :param snapshot_id: The snapshot to delete.
        :param delete_info: Volume backend technology specific data needed to
               be able to complete the snapshot.  For example, in the case of
               qcow2 backed snapshots, this would include the file being
               merged, and the file being merged into (if appropriate).
        """
        raise NotImplementedError()

    def default_root_device_name(self, instance, image_meta, root_bdm):
        """Provide a default root device name for the driver."""
        raise NotImplementedError()

    def default_device_names_for_instance(self, instance, root_device_name,
                                          *block_device_lists):
        """Default the missing device names in the block device mapping."""
        raise NotImplementedError()

    def is_supported_fs_format(self, fs_type):
        """Check whether the file format is supported by this driver

        :param fs_type: the file system type to be checked,
                        the validate values are defined at disk API module.
        """
        # NOTE(jichenjc): Return False here so that every hypervisor
        #                 need to define their supported file system
        #                 type and implement this function at their
        #                 virt layer.
        return False

    def quiesce(self, context, instance, image_meta):
        """Quiesce the specified instance to prepare for snapshots.

        If the specified instance doesn't support quiescing,
        InstanceQuiesceNotSupported is raised. When it fails to quiesce by
        other errors (e.g. agent timeout), NovaException is raised.

        :param context:  request context
        :param instance: nova.objects.instance.Instance to be quiesced
        :param image_meta: image object returned by nova.image.glance that
                           defines the image from which this instance
                           was created
        """
        raise NotImplementedError()

    def unquiesce(self, context, instance, image_meta):
        """Unquiesce the specified instance after snapshots.

        If the specified instance doesn't support quiescing,
        InstanceQuiesceNotSupported is raised. When it fails to quiesce by
        other errors (e.g. agent timeout), NovaException is raised.

        :param context:  request context
        :param instance: nova.objects.instance.Instance to be unquiesced
        :param image_meta: image object returned by nova.image.glance that
                           defines the image from which this instance
                           was created
        """
        raise NotImplementedError()

    def _get_version(self):
        return CONF.pcenter.version

    def _get_instance_capabilities(self, cpu_arch=None):
        """Return supported instances for a node."""
        if not cpu_arch:
            return []
        return [(cpu_arch,
                 hvtype.BAREMETAL,
                 vm_mode.HVM)]

    def _update_network_detail(self, context, network_info):
        """get vlan ID from given network info

        :param network_info: network info array to build the instance
        """
        network_client = network_api.get_client(context)
        for network in network_info:
            network_uuid = network['network']['id']
            network_detail = network_client.show_network(network_uuid).get(
                'network') or {}
            network_type = network_detail.get('provider:network_type')
            if not network_type or network_type not in PCENTER_NETWORK_TYPE:
                raise exception.VirtualInterfaceCreateException
            network['network']['vlan'] = network_detail.get(
                'provider:segmentation_id')


    def _update_network_detail_interface(self, context, network_info):
        """get vlan ID from given network info

        :param network_info: network info array to build the instance
        """
        network_client = network_api.get_client(context)
	network_uuid = network_info['network']['id']
	network_detail = network_client.show_network(network_uuid).get(
			'network') or {}
	network_type = network_detail.get('provider:network_type')
	if not network_type or network_type not in PCENTER_NETWORK_TYPE:
	    raise exception.VirtualInterfaceCreateException
	network_info['network']['vlan'] = network_detail.get(
			'provider:segmentation_id')
	return network_info









