# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
# Copyright (c) 2010 Citrix Systems, Inc.
# Copyright (c) 2011 Piston Cloud Computing, Inc
# Copyright (c) 2012 University Of Minho
# (c) Copyright 2013 Hewlett-Packard Development Company, L.P.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Manages information about the host OS and hypervisor.

This class encapsulates a connection to the pcenter
daemon and provides certain higher level APIs around
the raw pcenter API. These APIs are then used by all
the other pcenter related classes
"""

import operator
import os
import socket
import sys
import threading
import math
import eventlet
from eventlet import greenio
from eventlet import greenthread
from eventlet import patcher
from eventlet import tpool
from oslo_config import cfg
#from oslo_log import log as logging
from nova.openstack.common import log as logging
from oslo_utils import excutils
from oslo_utils import importutils
from oslo_utils import units


from nova import context as nova_context
from nova import exception
from nova.i18n import _
from nova.i18n import _LE
from nova.i18n import _LI
from nova.i18n import _LW
from nova import rpc
from nova import utils
from nova.virt.pcenter import client



LOG = logging.getLogger(__name__)

native_socket = patcher.original('socket')
native_threading = patcher.original("threading")
native_Queue = patcher.original("Queue")

CONF = cfg.CONF
CONF.import_opt('host', 'nova.netconf')
CONF.import_opt('my_ip', 'nova.netconf')


class Host(object):

    def __init__(self, hostname, client):
        # import pcenter rest api here.
        self._caps = None
        self.hostname = hostname
        self.client = client
        self.host_info = self.update_host_info()

    def update_host_info(self):
        
        path = ('/host/%(hostname)s/resource' % {
            'hostname': self.hostname})
        response = self.client.get(path)
        status_code = response.status_code
        if status_code == 200 and response.json().get('code') == '2007' :
            return response.json()
        if status_code != 200 or response.json().get('code') != '0000': 
            msg = (_(
                'Error from pcenter while get host %(hostname)s info.'
                'Status code: %(status_code)d.'
                'error: %(error)s.'
                'Reason: %(reason)s ' % {'hostname': self.hostname,
                                          'status_code': status_code,
                                          'error': response.json().get('msg'),
                                          'reason': response.reason}))
            raise exception.NovaException(msg)
        result = response.json()
        return result

    def create_lpar(self, lpar_name, vcpu, memory, disk, vifs, image_name, admin_passwd):
        path = ('/vm/add')
        if admin_passwd == None:
            data = {'host': self.hostname,
                    'name': lpar_name,
                    'cpu': vcpu,
                    'memory': memory,
                    'disk': int(disk),
                    'networks': vifs,
                    'type': "inner",
                    'os': image_name}
        else:
            data = {'host': self.hostname,
                    'name': lpar_name,
                    'cpu': vcpu,
                    'memory': memory,
                    'disk': int(disk),
                    'networks': vifs,
                    'type': "inner",
                    'passwd': admin_passwd,
                    'os': image_name}

        response = self.client.post(path, data)
        if response.status_code != 201 or response.json().get('code') != '0000':
            msg = (_(
                'Error from pcenter while create lpar on host %(hostname)s.'
                'Status code: %(status_code)d.'
                'error: %(error)s.'
                'Reason: %(reason)s ' % {'hostname': self.hostname,
                                         'status_code': response.status_code,
                                         'error': response.json().get('msg'),
                                         'reason': response.reason}))
            raise exception.NovaException(msg)


    def destroy_lpar(self, lpar_name):
        path = ('/vm/%(lpar_name)s/del' % {
                'lpar_name': lpar_name})
        response = self.client.delete(path)

        if response.status_code != 200 :
            msg = (_(
                'Error from pcenter while destroy lpar %(lpar_name)s.'
                'Status code: %(status_code)d.'
                'Reason: %(reason)s ' % {'lpar_name': lpar_name,
                                          'status_code': response.status_code,
                                          'reason': response.reason}))
            raise exception.NovaException(msg)
        result = response.json()
        #(lingcloudpower) Lpar name is not exist in Pcenter, delete directly
        # Do not raise error in nova
        if result.get('code') == '3006':
            return
        if result.get('code') != '0000':
            raise exception.NovaException('%s' %result.get('msg'))

    def get_vcpu_total(self):
        return int(
            math.floor(float(self.host_info['totalCpu'])))

    def get_memory_mb_total(self):
        return int(self.host_info['totalMemory'])

    def get_local_gb_info(self):
        info = {'total': (int(self.host_info['totalDisk']))/1024,
                'used': (int(self.host_info['totalDisk']) - int(
                    self.host_info['usableDisk']))/1024,
                'free': (int(self.host_info['usableDisk']))/1024}
        return info

    def get_vcpu_used(self):
        return int(
            math.floor(
                float(self.host_info['totalCpu']))) - int(
            math.floor(float(self.host_info[
                'usableCpu'])))


    def get_memory_mb_used(self):
        return int(self.host_info['totalMemory']) - \
               int(self.host_info['usableMemory'])

    def get_cpu_info(self):
        info = dict()
        info['arch'] = 'ppc64'
        info['mode'] = 'internal'
        info['vendor'] = 'ibm'
        return info


